@@ -61,9 +61,8 @@ magconf_gconf_client_init (void)
 
     if (error != NULL)
     {
-	sru_warning (_("magconf:Failed to set notify:\n"));
-	sru_warning (_("Recommended to delete ~/.gconf and ~/.gconfd directories"));
-	g_error_free(error);
+	sru_warning (_(error->message));
+	g_error_free (error);
 	error = NULL;
 	return FALSE;
     }
@@ -215,7 +214,7 @@ magconf_setting_new (void)
     new_magnifier = ( Magnifier * ) g_new0 ( Magnifier, 1);
     
     if (!new_magnifier) 
-	sru_error (_("Unable to allocate memory"));
+	sru_error (_("Unable to allocate memory."));
     
     return new_magnifier ;
 }
@@ -552,7 +551,7 @@ magconf_save_zoomer_in_schema (const gchar *schema,
 
     gnopiconf_set_int_in_section (magnifier->border_width, path, 	
 				    MAGNIFIER_BORDER_WIDTH);
-    gnopiconf_set_int_in_section (magnifier->border_color, path, 	
+    gnopiconf_set_int_in_section ((gint)magnifier->border_color, path, 	
 				    MAGNIFIER_BORDER_COLOR);
 			
     gnopiconf_set_bool_in_section (magnifier->cursor_state, path, 	
@@ -615,15 +614,15 @@ magconf_changes_cb			(GConfClient	*client,
 	if (!entry->value)
 	    return;
 		
-	sru_debug ("magconf:magconf_changes_cb:Entry key:%s",gconf_entry_get_key(entry));
-	sru_debug ("magconf:magconf_changes_cb:Entry value type:%i",entry->value->type);
+	sru_debug ("magconf_changes_cb:Entry key:%s",gconf_entry_get_key(entry));
+	sru_debug ("magconf_changes_cb:Entry value type:%i",entry->value->type);
 	switch(entry->value->type)
 	{
-    	    case GCONF_VALUE_INT:    sru_debug ("magconf:magconf_changes_cb:Entry value:%i",gconf_value_get_int(entry->value));break;
-    	    case GCONF_VALUE_FLOAT:  sru_debug ("magconf:magconf_changes_cb:Entry value:%f",gconf_value_get_float(entry->value));break;
-    	    case GCONF_VALUE_STRING: sru_debug ("magconf:magconf_changes_cb:Entry value:%s",gconf_value_get_string(entry->value));break;
-    	    case GCONF_VALUE_BOOL:   sru_debug ("magconf:magconf_changes_cb:Entry value:%i",gconf_value_get_bool(entry->value));break;
-	    default: sru_debug ("magconf:magconf_changes_cb:Other Entry value:");break;	
+    	    case GCONF_VALUE_INT:    sru_debug ("magconf_changes_cb:Entry value:%i",gconf_value_get_int(entry->value));break;
+    	    case GCONF_VALUE_FLOAT:  sru_debug ("magconf_changes_cb:Entry value:%f",gconf_value_get_float(entry->value));break;
+    	    case GCONF_VALUE_STRING: sru_debug ("magconf_changes_cb:Entry value:%s",gconf_value_get_string(entry->value));break;
+    	    case GCONF_VALUE_BOOL:   sru_debug ("magconf_changes_cb:Entry value:%i",gconf_value_get_bool(entry->value));break;
+	    default: sru_debug ("magconf_changes_cb:Other Entry value:");break;	
 	}
 
 	if (strcmp (key, CONFIG_PATH MAGNIFIER_PATH MAGNIFIER_SCHEMA1 MAGNIFIER_ZOOM_FACTOR_LOCK) == 0)
