@@ -354,7 +354,7 @@ src_speech_set_voice_parameters (gchar *voice,
     if (! (driver && driver[0] && name && name[0] && volume && volume[0] && 
 	    rate && rate[0] && pitch && pitch[0]))
     {
-	sru_warning (_("incorect parameters for voice \"%s%s\""), driver, voice); 
+	sru_warning (_("Incorrect parameters for voice \"%s%s\""), driver, voice); 
     }
     else
     {
@@ -386,9 +386,7 @@ src_speech_send (gchar *spcoutput)
     if (busy)
 	return FALSE;
     busy = TRUE;
-    /*FIXME: next line must be valid, no folowing 2 */
-    /* rv = srs_xml_output (spcoutput, strlen(spcoutput)); */
-    srs_xml_output (spcoutput, g_utf8_strlen(spcoutput, -1));
+    srs_xml_output (spcoutput, strlen(spcoutput)); 
     rv = TRUE;
     
 /*     fprintf (stderr, "\n%s", spcoutput); */
@@ -606,7 +604,7 @@ src_speech_test_voice (gchar *voice)
     if (rv)
     {
 	gchar *test;
-	test = g_strdup_printf (_("voice %s will sound this way"), voice);
+	test = g_strdup_printf (_("voice \"%s\" will sound this way"), voice);
 	rv = src_speech_say_ (test, "test", SRC_SPEECH_SPELL_NORMAL, TRUE);
 	g_free (test);
     }
@@ -892,43 +890,65 @@ static struct
     gchar  character;
     gchar *singular;
     gchar *plural;
-    SRCSpeechPunctuationType type;
+    gboolean translate;
 }src_speech_dictionary[]=
 {
-    {'~',	N_("Tilda"),		N_("Tildaes"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {'!',	N_("Exclamation"),	N_("Exclamations"),	SRC_SPEECH_PUNCTUATION_SAME},
-    {'@',	N_("At"),		N_("Ats"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {'#',	N_("Diez"),		N_("Diezes"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {'$',	N_("Dollar"),		N_("Dollars"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {'%',	N_("Percent"),		N_("Percents"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {'^',	N_("Caret"),		N_("Carets"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {'&',	N_("And"),		N_("Ands"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {'*',	N_("Asterix"),		N_("Asterixes"),	SRC_SPEECH_PUNCTUATION_SAME},
-    {'(',	N_("Left Paranthesises"),N_("Left Paranthesises"),SRC_SPEECH_PUNCTUATION_SAME},
-    {')',	N_("Right Paranthesises"),N_("Right Paranthesises"),SRC_SPEECH_PUNCTUATION_SAME},
-    {'_',	N_("Underscore"),	N_("Underscores"),	SRC_SPEECH_PUNCTUATION_SAME},
-    {'+',	N_("Plus"),		N_("Pluses"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {'=',	N_("Equal"),		N_("Equals"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {'-',	N_("Minus"),		N_("Minuses"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {'`',	N_("Quote"),		N_("Quotes"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {'[',	N_("Left Bracket"),	N_("Left Brackets"),	SRC_SPEECH_PUNCTUATION_SAME},
-    {']',	N_("Right Bracket"),	N_("Right Brackets"),	SRC_SPEECH_PUNCTUATION_SAME},
-    {'{',	N_("Left Brace"),	N_("Left Braces"),	SRC_SPEECH_PUNCTUATION_SAME},
-    {'}',	N_("Right Brace"),	N_("Right Braces"),	SRC_SPEECH_PUNCTUATION_SAME},
-    {';',	N_("Semicolon"),	N_("Semicolons"),	SRC_SPEECH_PUNCTUATION_SAME},
-    {':',	N_("Colon"),		N_("Colons"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {'"',	N_("Double quote"),	N_("Double quotes"),	SRC_SPEECH_PUNCTUATION_SAME},
-    {'\'',	N_("Apostrophe"),	N_("Apostrophes"),	SRC_SPEECH_PUNCTUATION_SAME},
-    {'\\',	N_("Back Slash"),	N_("Back Slashes"),	SRC_SPEECH_PUNCTUATION_SAME},
-    {'|',	N_("Bar"),		N_("Bars"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {'>',	N_("Grant"),		N_("Grants"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {'<',	N_("Less"),		N_("Lesses"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {',',	N_("Comma"),		N_("Commas"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {'.',	N_("Dot"),		N_("Dots"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {'/',	N_("Slash"),		N_("Slashes"),		SRC_SPEECH_PUNCTUATION_SAME},
-    {'?',	N_("Introgation"),	N_("Introgations"),	SRC_SPEECH_PUNCTUATION_SAME}
+    {'~',	N_("Tilde"),		N_("Tildaes"),		FALSE},
+    {'!',	N_("Exclamation"),	N_("Exclamations"),	FALSE},
+    {'@',	N_("At"),		N_("Ats"),		FALSE},
+    {'#',	N_("Hash"),		N_("Hashes"),		FALSE},
+    {'$',	N_("Dollar"),		N_("Dollars"),		FALSE},
+    {'%',	N_("Percent"),		N_("Percents"),		FALSE},
+    {'^',	N_("Caret"),		N_("Carets"),		FALSE},
+    {'&',	N_("And"),		N_("Ands"),		FALSE},
+    {'*',	N_("Asterisk"),		N_("Asterisks"),	FALSE},
+    {'(',	N_("Left Paranthesis"),N_("Left Paranthesis"),	FALSE},
+    {')',	N_("Right Paranthesis"),N_("Right Paranthesis"),FALSE},
+    {'_',	N_("Underscore"),	N_("Underscores"),	FALSE},
+    {'+',	N_("Plus"),		N_("Pluses"),		FALSE},
+    {'=',	N_("Equal"),		N_("Equals"),		FALSE},
+    {'-',	N_("Minus"),		N_("Minuses"),		FALSE},
+    {'`',	N_("Quote"),		N_("Quotes"),		FALSE},
+    {'[',	N_("Left Bracket"),	N_("Left Brackets"),	FALSE},
+    {']',	N_("Right Bracket"),	N_("Right Brackets"),	FALSE},
+    {'{',	N_("Left Brace"),	N_("Left Braces"),	FALSE},
+    {'}',	N_("Right Brace"),	N_("Right Braces"),	FALSE},
+    {';',	N_("Semicolon"),	N_("Semicolons"),	FALSE},
+    {':',	N_("Colon"),		N_("Colons"),		FALSE},
+    {'"',	N_("Double quote"),	N_("Double quotes"),	FALSE},
+    {'\'',	N_("Apostrophe"),	N_("Apostrophes"),	FALSE},
+    {'\\',	N_("Back Slash"),	N_("Back Slashes"),	FALSE},
+    {'|',	N_("Bar"),		N_("Bars"),		FALSE},
+    {'>',	N_("Greater"),		N_("Greaters"),		FALSE},
+    {'<',	N_("Less"),		N_("Lesses"),		FALSE},
+    {',',	N_("Comma"),		N_("Commas"),		FALSE},
+    {'.',	N_("Dot"),		N_("Dots"),		FALSE},
+    {'/',	N_("Slash"),		N_("Slashes"),		FALSE},
+    {'?',	N_("Question"),		N_("Questions"),	FALSE}
 };
 
+static gboolean
+src_speech_set_translate_for (gchar *set)
+{
+    gint i;
+
+    sru_assert (set);
+
+    for (i = 0; set[i]; i++)
+    {
+	gint j;
+	for (j = 0; j < G_N_ELEMENTS (src_speech_dictionary); j++)
+	{
+	    if (src_speech_dictionary[j].character == set[i])
+	    {
+		src_speech_dictionary[j].translate = TRUE;
+		break;
+	    }
+	}
+    }
+
+    return TRUE;
+}
 
 static gboolean
 src_speech_set_punctuation_type (SRCSpeechPunctuationType type)
@@ -937,7 +957,24 @@ src_speech_set_punctuation_type (SRCSpeechPunctuationType type)
     sru_assert (src_speech_initialized);
 
     for (i = 0; i < G_N_ELEMENTS (src_speech_dictionary); i++)
-	src_speech_dictionary[i]. type = type;
+	src_speech_dictionary[i]. translate = FALSE;
+
+    switch (type)
+    {
+	case SRC_SPEECH_PUNCTUATION_IGNORE:
+	    src_speech_set_translate_for ("");
+	    break;
+	case SRC_SPEECH_PUNCTUATION_SOME:
+	    src_speech_set_translate_for ("<>(){}*-+=");
+	    break;
+	case SRC_SPEECH_PUNCTUATION_MOST:
+	    src_speech_set_translate_for ("<>(){}*-+=.,!?\"'");
+	    break;
+	case SRC_SPEECH_PUNCTUATION_ALL:
+	    src_speech_set_translate_for ("<>(){}*-+=.,!?\"'&_[]|\\/");
+	    break;
+
+    }
 
     return TRUE;
 }
@@ -947,18 +984,18 @@ static gboolean
 src_speech_set_punctuation_type_from_string (gchar *type)
 {
     gboolean rv = TRUE;
-    SRCSpeechPunctuationType type_ = SRC_SPEECH_PUNCTUATION_SAME;
+    SRCSpeechPunctuationType type_ = SRC_SPEECH_PUNCTUATION_SOME;
 
     sru_assert (type);
 
     if (strcmp (type, "IGNORE") == 0)
 	type_ = SRC_SPEECH_PUNCTUATION_IGNORE;
-    else if (strcmp (type, "SAME") == 0)
-	type_ = SRC_SPEECH_PUNCTUATION_SAME;
-    else if (strcmp (type, "TRANSLATE") == 0)
-	type_ = SRC_SPEECH_PUNCTUATION_TRANSLATE;
-    else if (strcmp (type, "BOTH") == 0)
-	type_ = SRC_SPEECH_PUNCTUATION_BOTH;
+    else if (strcmp (type, "SOME") == 0)
+	type_ = SRC_SPEECH_PUNCTUATION_SOME;
+    else if (strcmp (type, "MOST") == 0)
+	type_ = SRC_SPEECH_PUNCTUATION_MOST;
+    else if (strcmp (type, "ALL") == 0)
+	type_ = SRC_SPEECH_PUNCTUATION_ALL;
     else
     {
 	sru_warning (_("value not supported for speech punctuation type \"%s\""), type);
@@ -1191,7 +1228,7 @@ src_speech_load_defaults ()
     if (tmp)
 	src_speech_set_punctuation_type_from_string (tmp);
     else
-        src_speech_set_punctuation_type (SRC_SPEECH_PUNCTUATION_SAME);
+        src_speech_set_punctuation_type (SRC_SPEECH_PUNCTUATION_SOME);
     g_free (tmp);
 
     tmp = NULL;
@@ -1399,31 +1436,23 @@ src_speech_translate_symbol (gint index)
 
     sru_assert (src_speech_initialized);
     sru_assert (index >= 0 && index < G_N_ELEMENTS (src_speech_dictionary));
-    
-    switch (src_speech_dictionary[index].type)
+
+    if (src_speech_dictionary[index].translate)
     {
-	case SRC_SPEECH_PUNCTUATION_IGNORE:
-	    break;
-	case SRC_SPEECH_PUNCTUATION_SAME:
-	    rv = g_strdup (" ");
-	    rv[0] = src_speech_dictionary [index].character;
-	    break;
-	case SRC_SPEECH_PUNCTUATION_TRANSLATE:
-	    rv = g_strdup (src_speech_dictionary[index].singular);
-	    break;
-	case SRC_SPEECH_PUNCTUATION_BOTH:
-	    {
-		gchar *tmp = g_strdup (" ");
-		tmp[0] = src_speech_dictionary [index].character;
-		rv = g_strconcat (tmp, " ", 
-					src_speech_dictionary[index].singular,
-					NULL);
-		g_free (tmp);
-	    }
-	    break;
-	default:
-	    sru_assert_not_reached ();
-	    break;
+	if (src_speech_dictionary[index].character == '.' ||
+	    src_speech_dictionary[index].character == '!' ||
+	    src_speech_dictionary[index].character == '?')
+	{
+	    rv = g_strconcat ("  ", src_speech_dictionary[index].singular, " ", NULL);
+	    rv[0] = src_speech_dictionary[index].character;
+	}
+	else
+	    rv = g_strconcat (" ", src_speech_dictionary[index].singular, " ", NULL);
+    }
+    else
+    {
+	rv = g_strdup (" ");
+	rv[0] = src_speech_dictionary[index].character;
     }
     return rv;
 }		
@@ -1468,8 +1497,8 @@ src_speech_process_string (const gchar   *text,
 			   SRCSpeechSpellingMode smode)
 {
     gchar *crt = (gchar*) text;
-    gchar *rv;
-    gchar *tmp;
+    gchar *rv  = NULL;
+    gchar *tmp = NULL;
     
     sru_assert (src_speech_initialized);
     sru_assert (text);
@@ -1484,8 +1513,14 @@ src_speech_process_string (const gchar   *text,
 	
     if (text[0] == ' ' && text[1] == '\0' && smode == SRC_SPEECH_SPELL_NORMAL)
 	return g_strdup (_("space"));
-    if (text[0] && text[1] == '\0')
-	return g_strdup (text);
+    if (text[0] && !text[1])
+    {
+        gint index = src_speech_get_index_for_symbol (crt[0]);
+	if (index != -1)
+	    return g_strdup (src_speech_dictionary[index].singular);
+	else
+	    return g_strdup ((gchar*)text);
+    }
 
     if (src_speech_dictionary_type == SRC_SPEECH_DICTIONARY_YES)
     {
@@ -1592,7 +1627,7 @@ src_speech_modify_pitch (SRCModifyMode mode)
 	for (i = 0; voices[i]; i++)
 	{
 	    gchar *pitch_ = NULL;
-	    gint pitch;
+	    gint   pitch = DEFAULT_SPEECH_PITCH;
 	
 	    if (mode != SRC_MODIF_DEFAULT)
 	    {
@@ -1604,8 +1639,7 @@ src_speech_modify_pitch (SRCModifyMode mode)
 			    -SRC_SPEECH_PITCH_STEP : SRC_SPEECH_PITCH_STEP;
 		}	
 	    }
-	    else
-		pitch = DEFAULT_SPEECH_PITCH;
+
 	    pitch = MIN (MAX (pitch, MIN_SPEECH_PITCH), MAX_SPEECH_PITCH);
 	    pitch_ = g_strdup_printf ("%d", pitch);
 	    if (src_speech_set_voice_parameters (voices[i], NULL, NULL, NULL, NULL, pitch_))
@@ -1642,7 +1676,7 @@ src_speech_modify_rate (SRCModifyMode mode)
 	for (i = 0; voices[i]; i++)
 	{
 	    gchar *rate_ = NULL;
-	    gint rate;
+	    gint rate = DEFAULT_SPEECH_RATE;
 	
 	    if (mode != SRC_MODIF_DEFAULT)
 	    {
@@ -1654,8 +1688,7 @@ src_speech_modify_rate (SRCModifyMode mode)
 			    -SRC_SPEECH_RATE_STEP : SRC_SPEECH_RATE_STEP;
 		}	
 	    }
-	    else
-		rate = DEFAULT_SPEECH_RATE;
+
 	    rate = MIN (MAX (rate, MIN_SPEECH_RATE), MAX_SPEECH_RATE);
 	    rate_ = g_strdup_printf ("%d", rate);
 	    if (src_speech_set_voice_parameters (voices[i], NULL, NULL, NULL, rate_, NULL))
@@ -1693,7 +1726,7 @@ src_speech_modify_volume (SRCModifyMode mode)
 	for (i = 0; voices[i]; i++)
 	{
 	    gchar *volume_ = NULL;
-	    gint volume;
+	    gint volume = DEFAULT_SPEECH_VOLUME;
 	
 	    if (mode != SRC_MODIF_DEFAULT)
 	    {
@@ -1705,8 +1738,7 @@ src_speech_modify_volume (SRCModifyMode mode)
 			    -SRC_SPEECH_VOLUME_STEP : SRC_SPEECH_VOLUME_STEP;
 		}
 	    }
-	    else
-		volume = DEFAULT_SPEECH_VOLUME;
+
 	    volume = MIN (MAX (volume, MIN_SPEECH_VOLUME), MAX_SPEECH_VOLUME);
 	    volume_ = g_strdup_printf ("%d", volume);
 	    if (src_speech_set_voice_parameters (voices[i], NULL, NULL, volume_, NULL, NULL))
